! FSMBPASS.FOR
! 12-JUN-1998, David Mathog, Biology Division, Caltech
! 
! implement an indexed file to hold SMB passwords.
! data/commands are passed in/out of these routines from 
! C, so they are passed manipulated here as just an array of
! integers.
!
! records contain
!
! username        32 bytes
! uic              4 bytes
! acct_ctrl        4 bytes
! lanman_password 16 bytes
! WNT_password    16 bytes
! SUM             72 bytes = 18 i*4 = RECL
!
! subroutine fsmb_openfile(byte filename(), i*4 unum, i*4 status)
! subroutine fsmb_closefile(i*4 unum, i*4 status)
! subroutine fsmb_find_by_name(i*4 unum, i*4 name(8), i*4 data(18), i*4 status)
! subroutine fsmb_find_by_uic(i*4 unum, i*4 uic, i*4 data(18), i*4 status)
! subroutine fsmb_append_entry(i*4 unum, i*4 data(18), i*4 status)
! subroutine fsmb_modify_last_entry(i*4 unum, i*4 data(18), i*4 status)
! subroutine fsmb_delete_last_entry(i*4 unum, i*4 status)
!
! filename is passed as a NULL terminate byte string, length <=256 
! characters
! 
! name is passed as exactly 32 bytes stored the same way as in the record. 
! That should be padded left with spaces so that the right most character
! of the username is at position 32.
!
! open the named file for sharing.  It will create a new one if an old
! one does not exist.
!
	subroutine fsmb_openfile(filename,unum,status)

	implicit none
	integer*4 flen,unum,status
	byte      filename(*)
!
	integer*4      i
	character*256 cfilename
	character*1   c1
	byte          b1
	equivalence (b1,c1)
!
	b1=1
	flen=0
	do while(b1 .ne. 0 .and. flen .lt. 256)
	   flen = flen + 1
	   b1=filename(flen)
	   cfilename(flen:flen)=c1
	end do
	if(b1 .eq. 0)flen = flen - 1
	if(flen .lt. 0)then
	  status = -1000
	  return
	end if
!
	open(UNIT=unum, 
	1  FILE=cfilename(1:flen),
	1  ORGANIZATION='indexed',
	1  ACCESS='KEYED',
	1  RECORDTYPE='FIXED',
	1  FORM='UNFORMATTED',
	1  RECL=18,
	1  KEY=(1:32:CHARACTER, 33:36:INTEGER),
	1  SHARED,
	1  STATUS='UNKNOWN',
	1  IOSTAT=STATUS)

	if(STATUS .ne. 0)then
	  status = -2
	end if

	return
	end


	subroutine fsmb_closefile(unum, status)
	implicit none
	integer*4 unum,status
!
	close(unit=unum,iostat=status)
	return
	end

	subroutine fsmb_find_by_name(unum, name, data, status)
	implicit none
	integer*4 unum, status, name(8), data(18)
!
	character*32 cname
	character*4 c4
	integer*4   i,j,i4
	equivalence (i4,c4(1:1))
!
! convert the array of integers into a valid character string
!
	j=1
	do i=1,8
	  i4=name(i)
	  cname(j:j+3)=c4
	  j = j + 4
	end do
!	
	read(unit=unum, KEY=cname, KEYID=0, IOSTAT=status)data
	return
	end

	subroutine fsmb_find_by_uic(unum, uic, data, status)
	implicit none
	integer*4 unum, status, uic, data(18)
!
	read(unit=unum, KEY=uic, KEYID=1, IOSTAT=status)data
	return
	end

	subroutine fsmb_append_entry(unum, data, status)
	implicit none
	integer*4 unum, status, data(18)
!
	write(unit=unum, IOSTAT=status)data
	return
	end

	subroutine fsmb_modify_last_entry(unum, data, status)
	implicit none
	integer*4 unum, status, data(18)
!
	rewrite(unit=unum, IOSTAT=status)data
	return
	end

	subroutine fsmb_delete_last_entry(unum, status)
	implicit none
	integer*4 unum, status
!
	delete(unit=unum, IOSTAT=status)
	return
	end
