/*
   Unix SMB/Netbios implementation.
   Version 1.9.
   Copyright (C) Andrew Tridgell 1994-1997

   This file is part of the port to OpenVMS
   Emulation routines
   Copyright (C) Eckart Meyer 1996-1997

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*	get.c
 *	V1.1			23-Nov-1995	IfN/Mey
 *
 *	Copyright E. Meyer <meyer@ifn.ing.tu-bs.de>
 *+
 * Some VMS specific routines using the various system services
 * and/or runtime libraries of VMS.
 *
 * NOTE: includes.h is *NOT* - and must not! - included here!
 */
/* 05-Jul-1998	J. Malmberg	Fixed to compile with GCC */
#ifndef __GNUC__	/* Why is this here? DECC supports const */
#define const
#endif
#include <stdio.h>
#include <iodef.h>
#include <ssdef.h>
#include <syidef.h>
#include <jpidef.h>
#include <brkdef.h>
#ifndef __GNUC__
#include <starlet.h>
#endif
#include <descrip.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#ifndef __GNUC__
#include <sys/timeb.h>
#else
#include <timeb.h>
#endif
#include <stdlib.h>
#ifdef __GNUC__
#include <unistd.h>
#else
#include <unixio.h>
#endif
#include <grp.h>
#include <pwd.h>

#include "longfile_vms.h"

int decc$get_sdc(int __descrip_no);

struct itemdesc {
  short int buffer_length;
  short int item_code;
  char *buffer;
  short int *ret_length;
  };

 /* Fix prototypes to actually do something usefull */
/*-------------------------------------------------*/

#ifdef __GNUC__
 /* Prototype for sys$getsyiw() */
/*---------------------------*/
unsigned long sys$getsyiw
       (unsigned long efn,
	unsigned long * csidadr,
	const struct dsc$descriptor_s * nodename,
	const struct itemdesc * itmlst,
	void * iosb,
	void (* astadr)(),
	unsigned long astprm);

unsigned long sys$idtoasc
       (unsigned long id,
	unsigned short * namlen,
	const struct dsc$descriptor_s * nambuf,
	unsigned long * resid,
	unsigned long * attrib,
	unsigned long * contxt);

unsigned long sys$asctoid
       (const struct dsc$descriptor_s * name,
	unsigned long * rights_id,
	unsigned long * attrib);

unsigned long sys$assign
       (const struct dsc$descriptor_s * devnam,
	unsigned short * chan,
	unsigned long acmode,
	const struct dsc$descriptor_s * mbxnam,
	unsigned long flags);

unsigned long sys$qiow
       (unsigned long efn,
	unsigned short chan,	/* This is right, 7.0 doc is wrong */
	unsigned long func,
	void * iosb,
	void (* astadr)(),
	...);
#endif

#if 1	/* for use with samba's debugging */
extern int DEBUGLEVEL;
#define DEBUG(level,body) ((DEBUGLEVEL>=(level))?(Debug1 body):0)
int Debug1();
#else
int DEBUGLEVEL = 8;
#define DEBUG(level,body) ((DEBUGLEVEL>=(level))?(printf body):0)
#endif

 /* For terminal input */
/*--------------------*/
static $DESCRIPTOR(termdesc, "SYS$INPUT:");
static unsigned short termchan = 0xffff;


#if 0
char *vms_make_name(char * __name);
char *vms_get_name(char * __name);
#endif

# if (__CRTL_VER < 70000000)
unsigned long vms_getuid()
{
#ifndef __GNUC__
  int getuid();
  int getgid();
#endif
  unsigned long uic;

  uic = (getgid() << 16) + getuid();
  if (uic == 0x00000000L || uic == 0x00010000L || uic == 0x00010004L) {
	uic = 0;	/* Samba need SYSTEM to be 0,0 ... */
  }
/* DEBUG (3,("vms_getuid: uid = %08X\n",uic)); */
  return(uic);
}
#else
unsigned long vms_getuid()
{
#ifndef __GNUC__
  int getuid();
#endif
  unsigned long uic;

  uic = getuid(); /* 7.0 and up return full UIC, not just user part */
  if (uic == 0x00000000L || uic == 0x00010000L || uic == 0x00010004L) {
	uic = 0;	/* Samba need SYSTEM to be 0,0 ... */
  }
/* DEBUG (3,("vms_getuid: uid = %08X\n",uic)); */
  return(uic);
}
#endif


 /* Get the password from sys$input */
/*=================================*/
 /* VMS allows 39 characters */
 /* Windows NT 4.0 allows 14 characters */
static char pwd[40];
char *getpass(const char * prompt)
{
unsigned long call_stat;
int tty;
unsigned long funct;
int plen;
int pwlen;
unsigned short iosb[4];

    pwd[0] = '\0';
    tty = isatty(STDIN_FILENO);

     /* This is only meaningfull for TTY sessions */
    /*-------------------------------------------*/
    if (tty)
    {
	 /* Look up the existing channel */
	/*------------------------------*/
	if (termchan == 0xffff)
	    call_stat = sys$assign( &termdesc, &termchan, 0, 0, 0);

	 /* Need a leading line feed */
	/*--------------------------*/
	putchar('\n');

	 /* Read the password with prompt */
	/*-------------------------------*/
	plen = strlen(prompt);
	pwlen = sizeof(pwd) - 1;
	funct = IO$_READPROMPT | IO$M_NOECHO | IO$M_PURGE;
	call_stat = sys$qiow
		       (0,
			termchan,
			funct,
			iosb,
			0,
			0,
			pwd,
			pwlen,
			0,
			0,
			prompt,
			plen);

	 /* Check the result */
	/*------------------*/
	if ((call_stat & SS$_NORMAL) == SS$_NORMAL)
	{
	    if ((iosb[0] & SS$_NORMAL) == SS$_NORMAL)
	    {
		 /* Null terminate the given password */
		/*-----------------------------------*/
		pwd[iosb[1]] = '\0';
	    }
	    else
		pwd[0] = '\0';
	}
	else
	    pwd[0] = '\0';

	 /* Fake the echo of the carriage return */
	/*--------------------------------------*/
	putchar('\r');
    }
    return(pwd);

}


char *vms_getcwd(char *path, size_t size)
{
#ifndef __GNUC__
char *getcwd(char * __path, size_t __size, int __flag);
#endif

char buf[256];
char * rptr;

    getcwd(path, size, 1);
    vms_to_longfile(buf, path, LONGFILE_M_UNIX);

    rptr = strncpy(path, buf, size);

    return rptr;
}

#if 0
int vms_getdechostname(char *name, int namelen)
{

struct itemdesc id[2];
unsigned short int iosb[4];
unsigned long call_stat;
short lgth;
int i;

   id[0].item_code = SYI$_NODENAME;
   id[0].buffer_length = namelen;
   id[0]buffer = name;
   id[0]ret_length = &lgth;
   id[0].item_code = 0;
   id[0].buffer_length = 0;

   call_stat = sys$getsyiw(0, 0, 0, id, iosb, 0, 0);
   if ((call_stat & SS$_NORMAL) != SS$_NORMAL)
	return(call_stat);

   name[lgth] = '\0';

   for (i=0;i < lgth;i++)
	if ((name[i] >= 'A') && (name[i] <= 'Z'))
	    name[i] += 'a'-'A';
   return(0);
}


int vms_getusertty(char *name, char *tty)
{
struct {
      struct itemdesc id[3];
      int eol;
   } itmlst;

unsigned short int iosb[4];
char uname[12];
char sname[12];
char stty[7];
char *p;
char *q;
short int i
short st;
short lgth1;
short lgth2;
short lgth3;
unsigned long pid;
int mode;

    for (p=name,q=uname;*p != '\0';p++,q++)
	if ((*p >= 'a') && (*p <= 'z'))
	    *q = *p-'a'+'A';
	else
	    *q = *p;
    for (i=0;i<12-strlen(p);i++,q++)
	*q = ' ';

   itmlst.id[0].item_code = JPI$_TERMINAL;
   itmlst.id[0].buffer_length = 7;
   itmlst.id[0].buffer = stty;
   itmlst.id[0].ret_length = &lgth1;
   itmlst.id[1].item_code = JPI$_USERNAME;
   itmlst.id[1].buffer_length = 12;
   itmlst.id[1].buffer = sname;
   itmlst.id[1].ret_length = &lgth2;
   itmlst.id[2].item_code = JPI$_MODE;
   itmlst.id[2].buffer_length = 4;
   itmlst.id[2].buffer = (char *)&mode;
   itmlst.id[2].ret_length = &lgth3;
   itmlst.eol = 0;
   pid = -1;
    while (1)
    {
 	if ((st=sys$getjpiw(0,&pid,0,&itmlst,iosb,0,0)) == SS$_NOMOREPROC) {
         return(-1);
      }
      if ((st == SS$_NORMAL) && (strncmp(sname,uname,12) == 0) && (mode == 3)) {
         strncpy(tty,stty,lgth1);
         tty[lgth1] = '\0';
         return(0);
      }
   }
}
#endif

#if 0
int vms_sendmsguser(char *s, char *t)
{
   unsigned short int iosb[4];
   short int st, i;
   char upname[16], *p;
   struct {
      int length;
      char *ptr;
   } msgdesc, senddesc;
   strcpy(upname,t);
   for (p=upname;*p != '\0';p++) if ((*p >= 'a') && (*p <= 'z')) *p -= 'a'-'A';
   msgdesc.length = strlen(s);
   msgdesc.ptr = s;
   senddesc.length = strlen(upname);
   senddesc.ptr = upname;
   if ((st= sys$brkthruw(0,&msgdesc,&senddesc,BRK$C_USERNAME,iosb,0,0,0,0,0,0))
    != SS$_NORMAL) {
      return(-1);
   }
   return(0);
}

int vms_sendmsgtty(char *s, char *t)
{
   unsigned short int iosb[4];
   short int st;
   struct {
      int length;
      char *ptr;
   } msgdesc, senddesc;
   msgdesc.length = strlen(s);
   msgdesc.ptr = s;
   senddesc.length = strlen(t);
   senddesc.ptr = t;
   if ((st= sys$brkthruw(0,&msgdesc,&senddesc,BRK$C_DEVICE,iosb,0,0,0,0,0,0))
    != SS$_NORMAL) {
      return(-1);
   }
   return(0);
}

int vms_getdtablesize()
{
   return(30);
}

int vms_getpagesize()
{
   return(512);
}
#endif

#if 0
struct group *vms_getgrent()
{
   return(NULL);
}
#endif

static struct group user_grp;
static char gr_name[32];
struct group *vms_getgrgid(gid_t gid)
{
struct dsc$descriptor_s dsc_name;
unsigned short namlen;
unsigned long sts;
unsigned long id;

    dsc_name.dsc$a_pointer = gr_name;
    dsc_name.dsc$w_length = sizeof(gr_name) - 1;
    dsc_name.dsc$b_class = DSC$K_CLASS_S;
    dsc_name.dsc$b_dtype = DSC$K_DTYPE_T;

    id = ((gid & 0xFFFF) << 16) | 0xFFFF;

    sts = sys$idtoasc(id,&namlen,&dsc_name,0,0,0);

    if (!(sts & SS$_NORMAL))
	return(NULL);

    gr_name[namlen] = '\0';
    user_grp.gr_name = gr_name;
    user_grp.gr_passwd = NULL;	/* not supported */
    user_grp.gr_gid = (int)gid;
    user_grp.gr_mem = NULL;		/* not supported */
    return(&user_grp);
}


static struct group user_grp1;
static char gr_name1[32];

struct group *vms_getgrnam(const char *name)
{
struct dsc$descriptor_s name_str;
unsigned long call_stat;
unsigned long right;
unsigned long attr;

    (const char *)name_str.dsc$a_pointer = name;
    name_str.dsc$w_length = sizeof(name) - 1;
    name_str.dsc$b_class = DSC$K_CLASS_S;
    name_str.dsc$b_dtype = DSC$K_DTYPE_T;

    call_stat = sys$asctoid(&name_str, &right, &attr);

    if ((call_stat & SS$_NORMAL) != SS$_NORMAL)
	return(NULL);

    strcpy(gr_name1, name);
    user_grp1.gr_name = gr_name1;
    user_grp1.gr_passwd = NULL;		/* not supported */
    user_grp1.gr_gid = (gid_t)((right & 0xFFFF) << 16) | 0xFFFF;
    user_grp1.gr_mem = NULL;		/* not supported */
    return(&user_grp1);

    return(NULL);
}

#if 0
void vms_setgrent()
{
}

void vms_endgrent()
{
}
#endif

static gid_t user_gid;
void vms_initgroups(const char *name, gid_t basegid)
{
	user_gid = basegid;
}

int vms_getgroups(int gidsetsize, gid_t grouplist[])
{
	if (gidsetsize > 0) {
		grouplist[0] = user_gid;
	}
	return(1);
}
#if 0
#include <iodef.h>
#include <ttdef.h>
#include <descrip.h>


/************************************************************/
/* set_noecho - just a utility routine to cancel term echo  */
/************************************************************/
void set_noecho()
{
  int st;
  int buf[3];
  short int iosb[4];

  if (termchan == -1) {
     sys$assign(&termdesc,&termchan,0,0);
  }
  st = sys$qiow(0,termchan,IO$_SENSEMODE,iosb,0,0,buf,12,0,0,0,0);
  buf[1] |= TT$M_NOECHO;
  st = sys$qiow(0,termchan,IO$_SETMODE,iosb,0,0,buf,12,0,0,0,0);
}

/************************************************************/
/* set_echo - just a utility routine to start term echo     */
/************************************************************/
void set_echo()
{
  int st;
  int buf[3];
  short int iosb[4];

  if (termchan == -1) {
     sys$assign(&termdesc,&termchan,0,0);
  }
  st = sys$qiow(0,termchan,IO$_SENSEMODE,iosb,0,0,buf,12,0,0,0,0);
  buf[1] &= ~TT$M_NOECHO;
  st = sys$qiow(0,termchan,IO$_SETMODE,iosb,0,0,buf,12,0,0,0,0);
}

#endif
