/*
   Unix SMB/Netbios implementation.
   Version 1.9.
   Copyright (C) Andrew Tridgell 1994-1997

   This file is part of the port to OpenVMS
   Privileged code
   Copyright (C) Eckart Meyer 1996-1997

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*	priv.c
 *	V1.0			6-Aug-1995	IfN/Mey
 *
 *	Copyright E. Meyer <meyer@ifn.ing.tu-bs.de>
 *+
 * Privileged code.
 *-
 */

/* 09-Jul-1998	J. Malmberg	Updated for GCC.  Added system prototypes
**		     *		Changed to use sys$hash_password
**
****************************************************************************/

#ifndef __GNUC__
#define EXTERN
#endif
#include "priv.h"

unsigned char login_encrypt;
unsigned short login_salt;
unsigned long login_expire[2];
unsigned long login_flags;
unsigned char login_net_acc_pri[3];
unsigned char login_net_acc_sec[3];
unsigned long login_primedays;
unsigned char login_pwd_len;
unsigned long login_userdata[255];

#include <descrip.h>
#include <ssdef.h>


 /* Prototypes for System Routines */
/*--------------------------------*/

#ifdef __GNUC__

 /* Hash password routine */
/*-----------------------*/
#ifdef sys$hash_password
#undef sys$hash_password
#endif
unsigned long sys$hash_password
       (const struct dsc$descriptor_s *pwd,
	const unsigned char alg,
	const unsigned short salt,
	const struct dsc$descriptor_s *usrnam,
	unsigned short *hash);

 /* Getuai routine */
/*----------------*/
#ifdef sys$getuai
#undef sys$getuai
#endif
unsigned long sys$getuai
       (unsigned long nullarg1,
	unsigned long * context,
	const struct dsc$descriptor_s *usrnam,
	const void * itmlst,
	unsigned long nullarg5,
	unsigned long nullarg6,
	unsigned long nullarg7);

 /* Find Holder Routine */
/*---------------------*/
#ifdef sys$find_held
#undef sys$find_held
#endif
unsigned long sys$find_held
       (const unsigned short *holder,
	unsigned long *id,
	unsigned long *attrib,
	unsigned long *context);

#ifdef sys$idtoasc
#undef sys$idtoasc
#endif
unsigned long sys$idtoasc
       (unsigned long id,
	unsigned short * namlen,
	struct dsc$descriptor_s * dsc_name,
	unsigned long * rsid,
	unsigned long * attib,
	unsigned long * contxt);
#endif

#if !defined(__DECC) && !defined(__GNUC__)
#define decc$translate_vms shell$translate_vms
#endif

char *decc$translate_vms(const char * __vmsfile);

static struct uai *uai_head = NULL;

char *vms_getlogin()
{
   char *cuserid(), *p;
   p = cuserid(login_name);
   for(;*p != '\0';p++) if ((*p >= 'A') && (*p <= 'Z')) *p += 'a'-'A';
   return(login_name);
}

struct uai *find_uai_by_name(char *name)
{
struct uai *puai;

    puai = uai_head;
    while (puai != NULL)
    {
	if (strcmp(name,puai->name) == 0)
	{
	    return(puai);
	}
	puai = puai->next;
    }
    return(NULL);
}

struct uai *find_uai_by_uic(unsigned long uic)
{
struct uai *puai;

    puai = uai_head;
    while (puai != NULL)
    {
	if (uic == puai->uic)
	{
	    return(puai);
	}
	puai = puai->next;
    }
    return(NULL);
}

static struct id *get_user_identifiers(unsigned long uic)
{
unsigned long ctx;
struct id *id_list;
struct id holder;
struct id id;
int count;
unsigned long vax_stat;

    holder.uic = uic;
    holder.attr = 0;

    id_list = (struct id *)malloc(sizeof(struct id));
    if (id_list == NULL)
	return(NULL);

    count = 0;
    ctx = 0;
    vax_stat = SS$_NORMAL;

    while ((vax_stat & SS$_NORMAL) == SS$_NORMAL )
    {
	vax_stat = sys$find_held
			((unsigned short *)&holder,
			 &id.uic,
			 &id.attr,
			 &ctx);

	id_list = (struct id *)realloc(id_list,sizeof(struct id) * (count+2) );
	if (id_list == NULL)
	    return(NULL);
	id_list[count].uic = id.uic;
	id_list[count].attr = id.attr;

	DEBUG(6,("  identifier %d = %08X\n",count+1,id.uic));

	count++;
    }

    id_list[count].uic = -1;

    DEBUG(6,("  uic %08X holds %d identifiers\n",uic,count));

    return(id_list);
}


#define UAI_CNT 13

 /* Get the proposed credentials for a later password check */
/*---------------------------------------------------------*/
long setup_pw()
{
unsigned long vax_stat;
/* unsigned short int iosb[4]; */
short lgth[UAI_CNT];

struct itemdesc id[UAI_CNT + 1];

struct dsc$descriptor_s namedesc;

char s[64];
char t[64];

struct uai *puai;

char *p;

   password_struct.pw_name = login_name;
   password_struct.pw_passwd = login_password;
#ifndef __GNUC__
   password_struct.pw_comment = login_comment;
#endif
   password_struct.pw_gecos = login_gecos;
   password_struct.pw_dir = login_dir;
   password_struct.pw_shell = login_shell;
   login_comment[0]  = '\0';
   login_gecos[0]    = '\0';
   login_shell[0]    = '\0';

   DEBUG (3,("setup_pw: username = \"%s\"\n", login_name));

   puai = find_uai_by_name(login_name);

   if (puai != NULL)
   {
	login_uai = puai;
	memcpy(login_password,puai->password,8);
	strcpy(login_dir,puai->dir);
	password_struct.pw_uid = puai->uic;
	password_struct.pw_gid = password_struct.pw_uid >> 16;

	login_priv[0] = puai->priv[0];
	login_priv[1] = puai->priv[1];

	DEBUG (3,("setup_pw: found uic = %08X\n", puai->uic));

	return(SS$_NORMAL);
   }

   id[0].item_code = UAI$_PWD;
   id[0].buffer_length = 8;
   id[0].buffer = login_password;
   id[0].ret_length = &lgth[0];

   id[1].item_code = UAI$_DEFDIR;
   id[1].buffer_length = 64;
   id[1].buffer = s;
   id[1].ret_length = &lgth[1];

   id[2].item_code = UAI$_DEFDEV;
   id[2].buffer_length = 64;
   id[2].buffer = t;
   id[2].ret_length = &lgth[2];

   id[3].item_code = UAI$_UIC;
   id[3].buffer_length = 4;
   id[3].buffer = (char *) &(password_struct.pw_uid);
   id[3].ret_length = &lgth[3];

   id[4].item_code = UAI$_PRIV;
   id[4].buffer_length = 8;
   id[4].buffer = (char *) &(login_priv[0]);
   id[4].ret_length = &lgth[4];

 /* 07-Jun-1998	J. Malmberg	Additional items needed for Password Test */
/*-----------------------------------------------------------------------*/
    id[5].item_code = UAI$_ENCRYPT;
    id[5].buffer_length = 1;
    id[5].buffer = (char *) &login_encrypt;
    id[5].ret_length = &lgth[5];

    id[6].item_code = UAI$_SALT;
    id[6].buffer_length = 2;
    id[6].buffer = (char *) &login_salt;
    id[6].ret_length = &lgth[6];

 /* 08-Jun-1998	J. Malmberg	Need flags, Network, account exp */
/*--------------------------------------------------------------*/
    id[7].item_code = UAI$_EXPIRATION;
    id[7].buffer_length = 8;
    id[7].buffer = (char *)login_expire;
    id[7].ret_length = &lgth[7];

    id[8].item_code = UAI$_FLAGS;
    id[8].buffer_length = 4;
    id[8].buffer = (char *)&login_flags;
    id[8].ret_length = &lgth[8];

 /* 09-Jun-1998	J. Malmberg	Also access permitted times to be complete */
/*------------------------------------------------------------------------*/
    id[9].item_code = UAI$_NETWORK_ACCESS_P;
    id[9].buffer_length = 3;
    id[9].buffer = (char *)login_net_acc_pri;
    id[9].ret_length = &lgth[9];

    id[10].item_code = UAI$_NETWORK_ACCESS_S;
    id[10].buffer_length = 3;
    id[10].buffer = (char *)login_net_acc_sec;
    id[10].ret_length = &lgth[10];

    id[11].item_code = UAI$_PRIMEDAYS;
    id[11].buffer_length = 4;
    id[11].buffer = (char *)&login_primedays;
    id[11].ret_length = &lgth[11];

 /* 11-Dec-1998	J. Malmberg	Get minimum password length for setting */
/*---------------------------------------------------------------------*/
    id[12].item_code = UAI$_PWD_LENGTH;
    id[12].buffer_length = 1;
    id[12].buffer = (char *)&login_pwd_len;
    id[12].ret_length = &lgth[12];

 /* End the request list */
/*----------------------*/
    id[UAI_CNT].item_code = 0;
    id[UAI_CNT].buffer_length = 0;

    namedesc.dsc$w_length = strlen(login_name);
    namedesc.dsc$a_pointer = login_name;
    namedesc.dsc$b_dtype = DSC$K_DTYPE_T;
    namedesc.dsc$b_class = DSC$K_CLASS_S;

    vax_stat = sys$getuai( 0, 0, &namedesc, id, 0, 0, 0);

    if ((vax_stat & SS$_NORMAL) != SS$_NORMAL)
    {
	DEBUG (3,("sys$getuai %08X %d\n", vax_stat, vax_stat));
	return(vax_stat);
    }

    password_struct.pw_gid = password_struct.pw_uid >> 16;
    login_password[8] = '\0';
    s[s[0]+1] = '\0';
    t[t[0]+1] = '\0';
    strcat(t+1,s+1);

    p = decc$translate_vms(t+1);
    if (p)
	strcpy(login_dir,p);

    DEBUG (5,("setup_pw: login dir = %s\n", login_dir));

    puai = (struct uai *)malloc(sizeof(struct uai));
    login_uai = puai;
    puai->uic = password_struct.pw_uid;
    puai->id_list = get_user_identifiers(puai->uic);
    puai->priv[0] = login_priv[0];
    puai->priv[1] = login_priv[1];
    strcpy(puai->dir,login_dir);
    memcpy(puai->password,login_password,8);
    strcpy(puai->name,login_name);
    if (uai_head == NULL)
    {
	puai->next = NULL;
	uai_head = puai;
    }
    else
    {
	puai->next = uai_head;
	uai_head = puai;
    }

    DEBUG (3,("setup_pw: $getuai returns UIC = %08X\n", puai->uic));
    DEBUG (3,("          priv = %08X %08X\n", puai->priv[0],puai->priv[1]));

    return(SS$_NORMAL);
}

  /* crypt routine.  VMS version needs the username, alg, and seed	 */
 /*  put in a global variables from the pw_setup routines		*/
/*---------------------------------------------------------------------*/
static char result[64];

char *crypt(char *key, char *salt)
{
unsigned long vax_stat;

struct dsc$descriptor_s fullnamedesc;
struct dsc$descriptor_s passwdesc;

    DEBUG (3,  ("crypt: username = \"%s\"\n",login_name));
    DEBUG (100,("crypt: key = \"%s\"\n",key));

    passwdesc.dsc$w_length = strlen(key);
    passwdesc.dsc$a_pointer = key;
    passwdesc.dsc$b_dtype = DSC$K_DTYPE_T;
    passwdesc.dsc$b_class = DSC$K_CLASS_S;

    fullnamedesc.dsc$w_length = strlen(login_name);
    fullnamedesc.dsc$a_pointer = login_name;
    fullnamedesc.dsc$b_dtype = DSC$K_DTYPE_T;
    fullnamedesc.dsc$b_class = DSC$K_CLASS_S;

    vax_stat = sys$hash_password
	       (&passwdesc,
		login_encrypt,
		login_salt,
		&fullnamedesc,
		(unsigned short *)result);
    if ((vax_stat & SS$_NORMAL) != SS$_NORMAL)
    {
	DEBUG (0,("crypt: sys$hash_password Error"));
    }
    result[8] = '\0';

    DEBUG (3,("crypt: result=%08X\n",*((int *)result)));
    DEBUG (3,("crypt: passwd=%08X\n",*((int *)login_password)));
    return(result);
}

struct passwd *vms_getpwnam(const char *name)
{
    char *p;
    unsigned long vax_stat;

    DEBUG (3,("getpwnam: name = \"%s\"\n",name));

    strcpy(login_name,name);
    for(p=login_name;*p != '\0';p++) *p = toupper(*p);

    vax_stat = setup_pw();

    if (vax_stat != SS$_NORMAL)
	return(NULL);
    return(&password_struct);
}

#ifdef __GNUC__
struct passwd *vms_getpwuid(unsigned long uid)
#else
struct passwd *vms_getpwuid(uid_t uid)
#endif
{
char *vms_getlogin();
struct uai *puai;
struct dsc$descriptor_s dsc_name;
unsigned short namlen;
unsigned long call_stat;

/*
 * If we have the entry, return it.
 * Otherwise use uid as an identifier and get the holder.
 */
   puai = find_uai_by_uic(uid);
   if (puai == NULL)
   {
	dsc_name.dsc$a_pointer = login_name;
	dsc_name.dsc$w_length = sizeof(login_name);
	dsc_name.dsc$b_class = DSC$K_CLASS_S;
	dsc_name.dsc$b_dtype = DSC$K_DTYPE_T;

	call_stat = sys$idtoasc
	    (uid,&namlen,&dsc_name,(void *)0, (void *)0, (void *)0);
	if ((call_stat & SS$_NORMAL) != SS$_NORMAL)
	    return(NULL);
	login_name[namlen] = '\0';
    }
    else
    {
	strcpy(login_name,puai->name);
    }

    call_stat = setup_pw();


    if (call_stat != SS$_NORMAL)
	return(NULL);
    return(&password_struct);
}

struct passwd *vms_getpwent()
{
char *vms_getlogin();
unsigned long vax_stat;

   password_struct.pw_name = vms_getlogin();

   vax_stat = setup_pw();
   if (vax_stat != SS$_NORMAL)
	return(NULL);
   return(&password_struct);
}

void vms_setpwent()
{
}

void vms_endpwent()
{
}


int vms_seteuid(uid_t euid)
{
   return(0);
}

int vms_setruid(uid_t ruid)
{
   return(0);
}

int vms_setgid(gid_t gid)
{
   return(0);
}
